#ifndef GAZEBO_COMMON_HH_
#define GAZEBO_COMMON_HH_

// Automatically generated
#include "gazebo/common/Animation.hh"
#include "gazebo/common/Assert.hh"
#include "gazebo/common/AudioDecoder.hh"
#include "gazebo/common/Battery.hh"
#include "gazebo/common/Base64.hh"
#include "gazebo/common/BVHLoader.hh"
#include "gazebo/common/ColladaLoader.hh"
#include "gazebo/common/CommonIface.hh"
#include "gazebo/common/CommonTypes.hh"
#include "gazebo/common/Console.hh"
#include "gazebo/common/Dem.hh"
#include "gazebo/common/EnumIface.hh"
#include "gazebo/common/Event.hh"
#include "gazebo/common/Events.hh"
#include "gazebo/common/Exception.hh"
#include "gazebo/common/FuelModelDatabase.hh"
#include "gazebo/common/MovingWindowFilter.hh"
#include "gazebo/common/HeightmapData.hh"
#include "gazebo/common/Image.hh"
#include "gazebo/common/ImageHeightmap.hh"
#include "gazebo/common/KeyEvent.hh"
#include "gazebo/common/KeyFrame.hh"
#include "gazebo/common/Material.hh"
#include "gazebo/common/MaterialDensity.hh"
#include "gazebo/common/Mesh.hh"
#include "gazebo/common/MeshLoader.hh"
#include "gazebo/common/MeshManager.hh"
#include "gazebo/common/ModelDatabase.hh"
#include "gazebo/common/MouseEvent.hh"
#include "gazebo/common/OBJLoader.hh"
#include "gazebo/common/PID.hh"
#include "gazebo/common/Plugin.hh"
#include "gazebo/common/SdfFrameSemantics.hh"
#include "gazebo/common/SemanticVersion.hh"
#include "gazebo/common/SkeletonAnimation.hh"
#include "gazebo/common/Skeleton.hh"
#include "gazebo/common/SingletonT.hh"
#include "gazebo/common/SphericalCoordinates.hh"
#include "gazebo/common/STLLoader.hh"
#include "gazebo/common/SystemPaths.hh"
#include "gazebo/common/SVGLoader.hh"
#include "gazebo/common/Time.hh"
#include "gazebo/common/Timer.hh"
#include "gazebo/common/UpdateInfo.hh"
#include "gazebo/common/URI.hh"
#include "gazebo/common/Video.hh"
#include "gazebo/common/VideoEncoder.hh"
#include "gazebo/common/WeakBind.hh"
#include "gazebo/common/ffmpeg_inc.h"


#endif
