syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface RaySensor 
/// \brief Information about a ray sensor element


message RaySensor
{
  optional bool display_scan            = 1;
  optional int32 horizontal_samples     = 2;
  optional double horizontal_resolution = 3;
  optional double horizontal_min_angle  = 4;
  optional double horizontal_max_angle  = 5;

  optional int32 vertical_samples       = 6;
  optional double vertical_resolution   = 7;
  optional double vertical_min_angle    = 8;
  optional double vertical_max_angle    = 9;

  optional double range_min             = 10;
  optional double range_max             = 11; 
  optional double range_resolution      = 12;
}

