// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: sensor_noise.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_sensor_5fnoise_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_sensor_5fnoise_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3020000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3020001 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/generated_enum_reflection.h>
#include <google/protobuf/unknown_field_set.h>
#ifndef _MSC_VER
#pragma GCC system_header
#pragma GCC diagnostic ignored "-Wshadow"
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif
#include <boost/shared_ptr.hpp>
#include <gazebo/util/system.hh>
#include "gazebo/msgs/MsgFactory.hh"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_sensor_5fnoise_2eproto GZ_MSGS_VISIBLE
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct GZ_MSGS_VISIBLE TableStruct_sensor_5fnoise_2eproto {
  static const uint32_t offsets[];
};
GZ_MSGS_VISIBLE extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_sensor_5fnoise_2eproto;
namespace gazebo {
namespace msgs {
class SensorNoise;
struct SensorNoiseDefaultTypeInternal;
GZ_MSGS_VISIBLE extern SensorNoiseDefaultTypeInternal _SensorNoise_default_instance_;
}  // namespace msgs
}  // namespace gazebo
PROTOBUF_NAMESPACE_OPEN
template<> GZ_MSGS_VISIBLE ::gazebo::msgs::SensorNoise* Arena::CreateMaybeMessage<::gazebo::msgs::SensorNoise>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace gazebo {
namespace msgs {

enum SensorNoise_Type : int {
  SensorNoise_Type_NONE = 1,
  SensorNoise_Type_GAUSSIAN = 2,
  SensorNoise_Type_GAUSSIAN_QUANTIZED = 3
};
GZ_MSGS_VISIBLE bool SensorNoise_Type_IsValid(int value);
constexpr SensorNoise_Type SensorNoise_Type_Type_MIN = SensorNoise_Type_NONE;
constexpr SensorNoise_Type SensorNoise_Type_Type_MAX = SensorNoise_Type_GAUSSIAN_QUANTIZED;
constexpr int SensorNoise_Type_Type_ARRAYSIZE = SensorNoise_Type_Type_MAX + 1;

GZ_MSGS_VISIBLE const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor* SensorNoise_Type_descriptor();
template<typename T>
inline const std::string& SensorNoise_Type_Name(T enum_t_value) {
  static_assert(::std::is_same<T, SensorNoise_Type>::value ||
    ::std::is_integral<T>::value,
    "Incorrect type passed to function SensorNoise_Type_Name.");
  return ::PROTOBUF_NAMESPACE_ID::internal::NameOfEnum(
    SensorNoise_Type_descriptor(), enum_t_value);
}
inline bool SensorNoise_Type_Parse(
    ::PROTOBUF_NAMESPACE_ID::ConstStringParam name, SensorNoise_Type* value) {
  return ::PROTOBUF_NAMESPACE_ID::internal::ParseNamedEnum<SensorNoise_Type>(
    SensorNoise_Type_descriptor(), name, value);
}
// ===================================================================

class GZ_MSGS_VISIBLE SensorNoise final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:gazebo.msgs.SensorNoise) */ {
 public:
  inline SensorNoise() : SensorNoise(nullptr) {}
  ~SensorNoise() override;
  explicit PROTOBUF_CONSTEXPR SensorNoise(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  SensorNoise(const SensorNoise& from);
  SensorNoise(SensorNoise&& from) noexcept
    : SensorNoise() {
    *this = ::std::move(from);
  }

  inline SensorNoise& operator=(const SensorNoise& from) {
    CopyFrom(from);
    return *this;
  }
  inline SensorNoise& operator=(SensorNoise&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const SensorNoise& default_instance() {
    return *internal_default_instance();
  }
  static inline const SensorNoise* internal_default_instance() {
    return reinterpret_cast<const SensorNoise*>(
               &_SensorNoise_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(SensorNoise& a, SensorNoise& b) {
    a.Swap(&b);
  }
  inline void Swap(SensorNoise* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SensorNoise* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  SensorNoise* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<SensorNoise>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const SensorNoise& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom(const SensorNoise& from);
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message* to, const ::PROTOBUF_NAMESPACE_ID::Message& from);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  void SharedCtor();
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(SensorNoise* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "gazebo.msgs.SensorNoise";
  }
  protected:
  explicit SensorNoise(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  typedef SensorNoise_Type Type;
  static constexpr Type NONE =
    SensorNoise_Type_NONE;
  static constexpr Type GAUSSIAN =
    SensorNoise_Type_GAUSSIAN;
  static constexpr Type GAUSSIAN_QUANTIZED =
    SensorNoise_Type_GAUSSIAN_QUANTIZED;
  static inline bool Type_IsValid(int value) {
    return SensorNoise_Type_IsValid(value);
  }
  static constexpr Type Type_MIN =
    SensorNoise_Type_Type_MIN;
  static constexpr Type Type_MAX =
    SensorNoise_Type_Type_MAX;
  static constexpr int Type_ARRAYSIZE =
    SensorNoise_Type_Type_ARRAYSIZE;
  static inline const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor*
  Type_descriptor() {
    return SensorNoise_Type_descriptor();
  }
  template<typename T>
  static inline const std::string& Type_Name(T enum_t_value) {
    static_assert(::std::is_same<T, Type>::value ||
      ::std::is_integral<T>::value,
      "Incorrect type passed to function Type_Name.");
    return SensorNoise_Type_Name(enum_t_value);
  }
  static inline bool Type_Parse(::PROTOBUF_NAMESPACE_ID::ConstStringParam name,
      Type* value) {
    return SensorNoise_Type_Parse(name, value);
  }

  // accessors -------------------------------------------------------

  enum : int {
    kMeanFieldNumber = 2,
    kStddevFieldNumber = 3,
    kBiasMeanFieldNumber = 4,
    kBiasStddevFieldNumber = 5,
    kPrecisionFieldNumber = 6,
    kTypeFieldNumber = 1,
  };
  // optional double mean = 2;
  bool has_mean() const;
  private:
  bool _internal_has_mean() const;
  public:
  void clear_mean();
  double mean() const;
  void set_mean(double value);
  private:
  double _internal_mean() const;
  void _internal_set_mean(double value);
  public:

  // optional double stddev = 3;
  bool has_stddev() const;
  private:
  bool _internal_has_stddev() const;
  public:
  void clear_stddev();
  double stddev() const;
  void set_stddev(double value);
  private:
  double _internal_stddev() const;
  void _internal_set_stddev(double value);
  public:

  // optional double bias_mean = 4;
  bool has_bias_mean() const;
  private:
  bool _internal_has_bias_mean() const;
  public:
  void clear_bias_mean();
  double bias_mean() const;
  void set_bias_mean(double value);
  private:
  double _internal_bias_mean() const;
  void _internal_set_bias_mean(double value);
  public:

  // optional double bias_stddev = 5;
  bool has_bias_stddev() const;
  private:
  bool _internal_has_bias_stddev() const;
  public:
  void clear_bias_stddev();
  double bias_stddev() const;
  void set_bias_stddev(double value);
  private:
  double _internal_bias_stddev() const;
  void _internal_set_bias_stddev(double value);
  public:

  // optional double precision = 6;
  bool has_precision() const;
  private:
  bool _internal_has_precision() const;
  public:
  void clear_precision();
  double precision() const;
  void set_precision(double value);
  private:
  double _internal_precision() const;
  void _internal_set_precision(double value);
  public:

  // required .gazebo.msgs.SensorNoise.Type type = 1;
  bool has_type() const;
  private:
  bool _internal_has_type() const;
  public:
  void clear_type();
  ::gazebo::msgs::SensorNoise_Type type() const;
  void set_type(::gazebo::msgs::SensorNoise_Type value);
  private:
  ::gazebo::msgs::SensorNoise_Type _internal_type() const;
  void _internal_set_type(::gazebo::msgs::SensorNoise_Type value);
  public:

  // @@protoc_insertion_point(class_scope:gazebo.msgs.SensorNoise)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  double mean_;
  double stddev_;
  double bias_mean_;
  double bias_stddev_;
  double precision_;
  int type_;
  friend struct ::TableStruct_sensor_5fnoise_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// SensorNoise

// required .gazebo.msgs.SensorNoise.Type type = 1;
inline bool SensorNoise::_internal_has_type() const {
  bool value = (_has_bits_[0] & 0x00000020u) != 0;
  return value;
}
inline bool SensorNoise::has_type() const {
  return _internal_has_type();
}
inline void SensorNoise::clear_type() {
  type_ = 1;
  _has_bits_[0] &= ~0x00000020u;
}
inline ::gazebo::msgs::SensorNoise_Type SensorNoise::_internal_type() const {
  return static_cast< ::gazebo::msgs::SensorNoise_Type >(type_);
}
inline ::gazebo::msgs::SensorNoise_Type SensorNoise::type() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.SensorNoise.type)
  return _internal_type();
}
inline void SensorNoise::_internal_set_type(::gazebo::msgs::SensorNoise_Type value) {
  assert(::gazebo::msgs::SensorNoise_Type_IsValid(value));
  _has_bits_[0] |= 0x00000020u;
  type_ = value;
}
inline void SensorNoise::set_type(::gazebo::msgs::SensorNoise_Type value) {
  _internal_set_type(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.SensorNoise.type)
}

// optional double mean = 2;
inline bool SensorNoise::_internal_has_mean() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool SensorNoise::has_mean() const {
  return _internal_has_mean();
}
inline void SensorNoise::clear_mean() {
  mean_ = 0;
  _has_bits_[0] &= ~0x00000001u;
}
inline double SensorNoise::_internal_mean() const {
  return mean_;
}
inline double SensorNoise::mean() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.SensorNoise.mean)
  return _internal_mean();
}
inline void SensorNoise::_internal_set_mean(double value) {
  _has_bits_[0] |= 0x00000001u;
  mean_ = value;
}
inline void SensorNoise::set_mean(double value) {
  _internal_set_mean(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.SensorNoise.mean)
}

// optional double stddev = 3;
inline bool SensorNoise::_internal_has_stddev() const {
  bool value = (_has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool SensorNoise::has_stddev() const {
  return _internal_has_stddev();
}
inline void SensorNoise::clear_stddev() {
  stddev_ = 0;
  _has_bits_[0] &= ~0x00000002u;
}
inline double SensorNoise::_internal_stddev() const {
  return stddev_;
}
inline double SensorNoise::stddev() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.SensorNoise.stddev)
  return _internal_stddev();
}
inline void SensorNoise::_internal_set_stddev(double value) {
  _has_bits_[0] |= 0x00000002u;
  stddev_ = value;
}
inline void SensorNoise::set_stddev(double value) {
  _internal_set_stddev(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.SensorNoise.stddev)
}

// optional double bias_mean = 4;
inline bool SensorNoise::_internal_has_bias_mean() const {
  bool value = (_has_bits_[0] & 0x00000004u) != 0;
  return value;
}
inline bool SensorNoise::has_bias_mean() const {
  return _internal_has_bias_mean();
}
inline void SensorNoise::clear_bias_mean() {
  bias_mean_ = 0;
  _has_bits_[0] &= ~0x00000004u;
}
inline double SensorNoise::_internal_bias_mean() const {
  return bias_mean_;
}
inline double SensorNoise::bias_mean() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.SensorNoise.bias_mean)
  return _internal_bias_mean();
}
inline void SensorNoise::_internal_set_bias_mean(double value) {
  _has_bits_[0] |= 0x00000004u;
  bias_mean_ = value;
}
inline void SensorNoise::set_bias_mean(double value) {
  _internal_set_bias_mean(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.SensorNoise.bias_mean)
}

// optional double bias_stddev = 5;
inline bool SensorNoise::_internal_has_bias_stddev() const {
  bool value = (_has_bits_[0] & 0x00000008u) != 0;
  return value;
}
inline bool SensorNoise::has_bias_stddev() const {
  return _internal_has_bias_stddev();
}
inline void SensorNoise::clear_bias_stddev() {
  bias_stddev_ = 0;
  _has_bits_[0] &= ~0x00000008u;
}
inline double SensorNoise::_internal_bias_stddev() const {
  return bias_stddev_;
}
inline double SensorNoise::bias_stddev() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.SensorNoise.bias_stddev)
  return _internal_bias_stddev();
}
inline void SensorNoise::_internal_set_bias_stddev(double value) {
  _has_bits_[0] |= 0x00000008u;
  bias_stddev_ = value;
}
inline void SensorNoise::set_bias_stddev(double value) {
  _internal_set_bias_stddev(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.SensorNoise.bias_stddev)
}

// optional double precision = 6;
inline bool SensorNoise::_internal_has_precision() const {
  bool value = (_has_bits_[0] & 0x00000010u) != 0;
  return value;
}
inline bool SensorNoise::has_precision() const {
  return _internal_has_precision();
}
inline void SensorNoise::clear_precision() {
  precision_ = 0;
  _has_bits_[0] &= ~0x00000010u;
}
inline double SensorNoise::_internal_precision() const {
  return precision_;
}
inline double SensorNoise::precision() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.SensorNoise.precision)
  return _internal_precision();
}
inline void SensorNoise::_internal_set_precision(double value) {
  _has_bits_[0] |= 0x00000010u;
  precision_ = value;
}
inline void SensorNoise::set_precision(double value) {
  _internal_set_precision(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.SensorNoise.precision)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

typedef boost::shared_ptr<gazebo::msgs::SensorNoise> SensorNoisePtr;
// @@protoc_insertion_point(namespace_scope)

}  // namespace msgs
}  // namespace gazebo

PROTOBUF_NAMESPACE_OPEN

template <> struct is_proto_enum< ::gazebo::msgs::SensorNoise_Type> : ::std::true_type {};
template <>
inline const EnumDescriptor* GetEnumDescriptor< ::gazebo::msgs::SensorNoise_Type>() {
  return ::gazebo::msgs::SensorNoise_Type_descriptor();
}

PROTOBUF_NAMESPACE_CLOSE

typedef const boost::shared_ptr<gazebo::msgs::SensorNoise const> ConstSensorNoisePtr;
// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_sensor_5fnoise_2eproto
