// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: joint_animation.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_joint_5fanimation_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_joint_5fanimation_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3020000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3020001 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "time.pb.h"
#ifndef _MSC_VER
#pragma GCC system_header
#pragma GCC diagnostic ignored "-Wshadow"
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif
#include <boost/shared_ptr.hpp>
#include <gazebo/util/system.hh>
#include "gazebo/msgs/MsgFactory.hh"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_joint_5fanimation_2eproto GZ_MSGS_VISIBLE
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct GZ_MSGS_VISIBLE TableStruct_joint_5fanimation_2eproto {
  static const uint32_t offsets[];
};
GZ_MSGS_VISIBLE extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_joint_5fanimation_2eproto;
namespace gazebo {
namespace msgs {
class JointAnimation;
struct JointAnimationDefaultTypeInternal;
GZ_MSGS_VISIBLE extern JointAnimationDefaultTypeInternal _JointAnimation_default_instance_;
class JointAnimation_Joint;
struct JointAnimation_JointDefaultTypeInternal;
GZ_MSGS_VISIBLE extern JointAnimation_JointDefaultTypeInternal _JointAnimation_Joint_default_instance_;
}  // namespace msgs
}  // namespace gazebo
PROTOBUF_NAMESPACE_OPEN
template<> GZ_MSGS_VISIBLE ::gazebo::msgs::JointAnimation* Arena::CreateMaybeMessage<::gazebo::msgs::JointAnimation>(Arena*);
template<> GZ_MSGS_VISIBLE ::gazebo::msgs::JointAnimation_Joint* Arena::CreateMaybeMessage<::gazebo::msgs::JointAnimation_Joint>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace gazebo {
namespace msgs {

// ===================================================================

class GZ_MSGS_VISIBLE JointAnimation_Joint final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:gazebo.msgs.JointAnimation.Joint) */ {
 public:
  inline JointAnimation_Joint() : JointAnimation_Joint(nullptr) {}
  ~JointAnimation_Joint() override;
  explicit PROTOBUF_CONSTEXPR JointAnimation_Joint(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  JointAnimation_Joint(const JointAnimation_Joint& from);
  JointAnimation_Joint(JointAnimation_Joint&& from) noexcept
    : JointAnimation_Joint() {
    *this = ::std::move(from);
  }

  inline JointAnimation_Joint& operator=(const JointAnimation_Joint& from) {
    CopyFrom(from);
    return *this;
  }
  inline JointAnimation_Joint& operator=(JointAnimation_Joint&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const JointAnimation_Joint& default_instance() {
    return *internal_default_instance();
  }
  static inline const JointAnimation_Joint* internal_default_instance() {
    return reinterpret_cast<const JointAnimation_Joint*>(
               &_JointAnimation_Joint_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(JointAnimation_Joint& a, JointAnimation_Joint& b) {
    a.Swap(&b);
  }
  inline void Swap(JointAnimation_Joint* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(JointAnimation_Joint* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  JointAnimation_Joint* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<JointAnimation_Joint>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const JointAnimation_Joint& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom(const JointAnimation_Joint& from);
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message* to, const ::PROTOBUF_NAMESPACE_ID::Message& from);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  void SharedCtor();
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(JointAnimation_Joint* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "gazebo.msgs.JointAnimation.Joint";
  }
  protected:
  explicit JointAnimation_Joint(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kNameFieldNumber = 1,
    kAngleFieldNumber = 2,
  };
  // repeated string name = 1;
  int name_size() const;
  private:
  int _internal_name_size() const;
  public:
  void clear_name();
  const std::string& name(int index) const;
  std::string* mutable_name(int index);
  void set_name(int index, const std::string& value);
  void set_name(int index, std::string&& value);
  void set_name(int index, const char* value);
  void set_name(int index, const char* value, size_t size);
  std::string* add_name();
  void add_name(const std::string& value);
  void add_name(std::string&& value);
  void add_name(const char* value);
  void add_name(const char* value, size_t size);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string>& name() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string>* mutable_name();
  private:
  const std::string& _internal_name(int index) const;
  std::string* _internal_add_name();
  public:

  // repeated double angle = 2;
  int angle_size() const;
  private:
  int _internal_angle_size() const;
  public:
  void clear_angle();
  private:
  double _internal_angle(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >&
      _internal_angle() const;
  void _internal_add_angle(double value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >*
      _internal_mutable_angle();
  public:
  double angle(int index) const;
  void set_angle(int index, double value);
  void add_angle(double value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >&
      angle() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >*
      mutable_angle();

  // @@protoc_insertion_point(class_scope:gazebo.msgs.JointAnimation.Joint)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string> name_;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< double > angle_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  friend struct ::TableStruct_joint_5fanimation_2eproto;
};
// -------------------------------------------------------------------

class GZ_MSGS_VISIBLE JointAnimation final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:gazebo.msgs.JointAnimation) */ {
 public:
  inline JointAnimation() : JointAnimation(nullptr) {}
  ~JointAnimation() override;
  explicit PROTOBUF_CONSTEXPR JointAnimation(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  JointAnimation(const JointAnimation& from);
  JointAnimation(JointAnimation&& from) noexcept
    : JointAnimation() {
    *this = ::std::move(from);
  }

  inline JointAnimation& operator=(const JointAnimation& from) {
    CopyFrom(from);
    return *this;
  }
  inline JointAnimation& operator=(JointAnimation&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const JointAnimation& default_instance() {
    return *internal_default_instance();
  }
  static inline const JointAnimation* internal_default_instance() {
    return reinterpret_cast<const JointAnimation*>(
               &_JointAnimation_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(JointAnimation& a, JointAnimation& b) {
    a.Swap(&b);
  }
  inline void Swap(JointAnimation* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(JointAnimation* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  JointAnimation* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<JointAnimation>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const JointAnimation& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom(const JointAnimation& from);
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message* to, const ::PROTOBUF_NAMESPACE_ID::Message& from);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  void SharedCtor();
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(JointAnimation* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "gazebo.msgs.JointAnimation";
  }
  protected:
  explicit JointAnimation(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  typedef JointAnimation_Joint Joint;

  // accessors -------------------------------------------------------

  enum : int {
    kJointFieldNumber = 2,
    kTimeFieldNumber = 3,
    kModelNameFieldNumber = 1,
  };
  // repeated .gazebo.msgs.JointAnimation.Joint joint = 2;
  int joint_size() const;
  private:
  int _internal_joint_size() const;
  public:
  void clear_joint();
  ::gazebo::msgs::JointAnimation_Joint* mutable_joint(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::JointAnimation_Joint >*
      mutable_joint();
  private:
  const ::gazebo::msgs::JointAnimation_Joint& _internal_joint(int index) const;
  ::gazebo::msgs::JointAnimation_Joint* _internal_add_joint();
  public:
  const ::gazebo::msgs::JointAnimation_Joint& joint(int index) const;
  ::gazebo::msgs::JointAnimation_Joint* add_joint();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::JointAnimation_Joint >&
      joint() const;

  // repeated .gazebo.msgs.Time time = 3;
  int time_size() const;
  private:
  int _internal_time_size() const;
  public:
  void clear_time();
  ::gazebo::msgs::Time* mutable_time(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Time >*
      mutable_time();
  private:
  const ::gazebo::msgs::Time& _internal_time(int index) const;
  ::gazebo::msgs::Time* _internal_add_time();
  public:
  const ::gazebo::msgs::Time& time(int index) const;
  ::gazebo::msgs::Time* add_time();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Time >&
      time() const;

  // required string model_name = 1;
  bool has_model_name() const;
  private:
  bool _internal_has_model_name() const;
  public:
  void clear_model_name();
  const std::string& model_name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_model_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_model_name();
  PROTOBUF_NODISCARD std::string* release_model_name();
  void set_allocated_model_name(std::string* model_name);
  private:
  const std::string& _internal_model_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_model_name(const std::string& value);
  std::string* _internal_mutable_model_name();
  public:

  // @@protoc_insertion_point(class_scope:gazebo.msgs.JointAnimation)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::JointAnimation_Joint > joint_;
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Time > time_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr model_name_;
  friend struct ::TableStruct_joint_5fanimation_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// JointAnimation_Joint

// repeated string name = 1;
inline int JointAnimation_Joint::_internal_name_size() const {
  return name_.size();
}
inline int JointAnimation_Joint::name_size() const {
  return _internal_name_size();
}
inline void JointAnimation_Joint::clear_name() {
  name_.Clear();
}
inline std::string* JointAnimation_Joint::add_name() {
  std::string* _s = _internal_add_name();
  // @@protoc_insertion_point(field_add_mutable:gazebo.msgs.JointAnimation.Joint.name)
  return _s;
}
inline const std::string& JointAnimation_Joint::_internal_name(int index) const {
  return name_.Get(index);
}
inline const std::string& JointAnimation_Joint::name(int index) const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.JointAnimation.Joint.name)
  return _internal_name(index);
}
inline std::string* JointAnimation_Joint::mutable_name(int index) {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.JointAnimation.Joint.name)
  return name_.Mutable(index);
}
inline void JointAnimation_Joint::set_name(int index, const std::string& value) {
  name_.Mutable(index)->assign(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.JointAnimation.Joint.name)
}
inline void JointAnimation_Joint::set_name(int index, std::string&& value) {
  name_.Mutable(index)->assign(std::move(value));
  // @@protoc_insertion_point(field_set:gazebo.msgs.JointAnimation.Joint.name)
}
inline void JointAnimation_Joint::set_name(int index, const char* value) {
  GOOGLE_DCHECK(value != nullptr);
  name_.Mutable(index)->assign(value);
  // @@protoc_insertion_point(field_set_char:gazebo.msgs.JointAnimation.Joint.name)
}
inline void JointAnimation_Joint::set_name(int index, const char* value, size_t size) {
  name_.Mutable(index)->assign(
    reinterpret_cast<const char*>(value), size);
  // @@protoc_insertion_point(field_set_pointer:gazebo.msgs.JointAnimation.Joint.name)
}
inline std::string* JointAnimation_Joint::_internal_add_name() {
  return name_.Add();
}
inline void JointAnimation_Joint::add_name(const std::string& value) {
  name_.Add()->assign(value);
  // @@protoc_insertion_point(field_add:gazebo.msgs.JointAnimation.Joint.name)
}
inline void JointAnimation_Joint::add_name(std::string&& value) {
  name_.Add(std::move(value));
  // @@protoc_insertion_point(field_add:gazebo.msgs.JointAnimation.Joint.name)
}
inline void JointAnimation_Joint::add_name(const char* value) {
  GOOGLE_DCHECK(value != nullptr);
  name_.Add()->assign(value);
  // @@protoc_insertion_point(field_add_char:gazebo.msgs.JointAnimation.Joint.name)
}
inline void JointAnimation_Joint::add_name(const char* value, size_t size) {
  name_.Add()->assign(reinterpret_cast<const char*>(value), size);
  // @@protoc_insertion_point(field_add_pointer:gazebo.msgs.JointAnimation.Joint.name)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string>&
JointAnimation_Joint::name() const {
  // @@protoc_insertion_point(field_list:gazebo.msgs.JointAnimation.Joint.name)
  return name_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string>*
JointAnimation_Joint::mutable_name() {
  // @@protoc_insertion_point(field_mutable_list:gazebo.msgs.JointAnimation.Joint.name)
  return &name_;
}

// repeated double angle = 2;
inline int JointAnimation_Joint::_internal_angle_size() const {
  return angle_.size();
}
inline int JointAnimation_Joint::angle_size() const {
  return _internal_angle_size();
}
inline void JointAnimation_Joint::clear_angle() {
  angle_.Clear();
}
inline double JointAnimation_Joint::_internal_angle(int index) const {
  return angle_.Get(index);
}
inline double JointAnimation_Joint::angle(int index) const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.JointAnimation.Joint.angle)
  return _internal_angle(index);
}
inline void JointAnimation_Joint::set_angle(int index, double value) {
  angle_.Set(index, value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.JointAnimation.Joint.angle)
}
inline void JointAnimation_Joint::_internal_add_angle(double value) {
  angle_.Add(value);
}
inline void JointAnimation_Joint::add_angle(double value) {
  _internal_add_angle(value);
  // @@protoc_insertion_point(field_add:gazebo.msgs.JointAnimation.Joint.angle)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >&
JointAnimation_Joint::_internal_angle() const {
  return angle_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >&
JointAnimation_Joint::angle() const {
  // @@protoc_insertion_point(field_list:gazebo.msgs.JointAnimation.Joint.angle)
  return _internal_angle();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >*
JointAnimation_Joint::_internal_mutable_angle() {
  return &angle_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >*
JointAnimation_Joint::mutable_angle() {
  // @@protoc_insertion_point(field_mutable_list:gazebo.msgs.JointAnimation.Joint.angle)
  return _internal_mutable_angle();
}

// -------------------------------------------------------------------

// JointAnimation

// required string model_name = 1;
inline bool JointAnimation::_internal_has_model_name() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool JointAnimation::has_model_name() const {
  return _internal_has_model_name();
}
inline void JointAnimation::clear_model_name() {
  model_name_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000001u;
}
inline const std::string& JointAnimation::model_name() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.JointAnimation.model_name)
  return _internal_model_name();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void JointAnimation::set_model_name(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000001u;
 model_name_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:gazebo.msgs.JointAnimation.model_name)
}
inline std::string* JointAnimation::mutable_model_name() {
  std::string* _s = _internal_mutable_model_name();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.JointAnimation.model_name)
  return _s;
}
inline const std::string& JointAnimation::_internal_model_name() const {
  return model_name_.Get();
}
inline void JointAnimation::_internal_set_model_name(const std::string& value) {
  _has_bits_[0] |= 0x00000001u;
  model_name_.Set(value, GetArenaForAllocation());
}
inline std::string* JointAnimation::_internal_mutable_model_name() {
  _has_bits_[0] |= 0x00000001u;
  return model_name_.Mutable(GetArenaForAllocation());
}
inline std::string* JointAnimation::release_model_name() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.JointAnimation.model_name)
  if (!_internal_has_model_name()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000001u;
  auto* p = model_name_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (model_name_.IsDefault()) {
    model_name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void JointAnimation::set_allocated_model_name(std::string* model_name) {
  if (model_name != nullptr) {
    _has_bits_[0] |= 0x00000001u;
  } else {
    _has_bits_[0] &= ~0x00000001u;
  }
  model_name_.SetAllocated(model_name, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (model_name_.IsDefault()) {
    model_name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.JointAnimation.model_name)
}

// repeated .gazebo.msgs.JointAnimation.Joint joint = 2;
inline int JointAnimation::_internal_joint_size() const {
  return joint_.size();
}
inline int JointAnimation::joint_size() const {
  return _internal_joint_size();
}
inline void JointAnimation::clear_joint() {
  joint_.Clear();
}
inline ::gazebo::msgs::JointAnimation_Joint* JointAnimation::mutable_joint(int index) {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.JointAnimation.joint)
  return joint_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::JointAnimation_Joint >*
JointAnimation::mutable_joint() {
  // @@protoc_insertion_point(field_mutable_list:gazebo.msgs.JointAnimation.joint)
  return &joint_;
}
inline const ::gazebo::msgs::JointAnimation_Joint& JointAnimation::_internal_joint(int index) const {
  return joint_.Get(index);
}
inline const ::gazebo::msgs::JointAnimation_Joint& JointAnimation::joint(int index) const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.JointAnimation.joint)
  return _internal_joint(index);
}
inline ::gazebo::msgs::JointAnimation_Joint* JointAnimation::_internal_add_joint() {
  return joint_.Add();
}
inline ::gazebo::msgs::JointAnimation_Joint* JointAnimation::add_joint() {
  ::gazebo::msgs::JointAnimation_Joint* _add = _internal_add_joint();
  // @@protoc_insertion_point(field_add:gazebo.msgs.JointAnimation.joint)
  return _add;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::JointAnimation_Joint >&
JointAnimation::joint() const {
  // @@protoc_insertion_point(field_list:gazebo.msgs.JointAnimation.joint)
  return joint_;
}

// repeated .gazebo.msgs.Time time = 3;
inline int JointAnimation::_internal_time_size() const {
  return time_.size();
}
inline int JointAnimation::time_size() const {
  return _internal_time_size();
}
inline ::gazebo::msgs::Time* JointAnimation::mutable_time(int index) {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.JointAnimation.time)
  return time_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Time >*
JointAnimation::mutable_time() {
  // @@protoc_insertion_point(field_mutable_list:gazebo.msgs.JointAnimation.time)
  return &time_;
}
inline const ::gazebo::msgs::Time& JointAnimation::_internal_time(int index) const {
  return time_.Get(index);
}
inline const ::gazebo::msgs::Time& JointAnimation::time(int index) const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.JointAnimation.time)
  return _internal_time(index);
}
inline ::gazebo::msgs::Time* JointAnimation::_internal_add_time() {
  return time_.Add();
}
inline ::gazebo::msgs::Time* JointAnimation::add_time() {
  ::gazebo::msgs::Time* _add = _internal_add_time();
  // @@protoc_insertion_point(field_add:gazebo.msgs.JointAnimation.time)
  return _add;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Time >&
JointAnimation::time() const {
  // @@protoc_insertion_point(field_list:gazebo.msgs.JointAnimation.time)
  return time_;
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------


typedef boost::shared_ptr<gazebo::msgs::JointAnimation> JointAnimationPtr;
// @@protoc_insertion_point(namespace_scope)

}  // namespace msgs
}  // namespace gazebo

typedef const boost::shared_ptr<gazebo::msgs::JointAnimation const> ConstJointAnimationPtr;
// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_joint_5fanimation_2eproto
